@@ -1,7 +1,9 @@
 from __future__ import annotations
 
+import sys
 from functools import partial
 from multiprocessing.synchronize import Event as EventType
+from random import randint
 from typing import Awaitable, Callable, Optional
 
 import trio
@@ -21,6 +23,9 @@ from ..utils import (
     ShutdownError,
 )
 
+if sys.version_info < (3, 11):
+    from exceptiongroup import BaseExceptionGroup
+
 
 async def worker_serve(
     app: AppWrapper,
@@ -33,7 +38,10 @@ async def worker_serve(
     config.set_statsd_logger_class(StatsdLogger)
 
     lifespan = Lifespan(app, config)
-    context = WorkerContext()
+    max_requests = None
+    if config.max_requests is not None:
+        max_requests = config.max_requests + randint(0, config.max_requests_jitter)
+    context = WorkerContext(max_requests)
 
     async with trio.open_nursery() as lifespan_nursery:
         await lifespan_nursery.start(lifespan.handle_lifespan)
@@ -75,9 +83,10 @@ async def worker_serve(
 
             task_status.started(binds)
             try:
-                async with trio.open_nursery() as nursery:
+                async with trio.open_nursery(strict_exception_groups=True) as nursery:
                     if shutdown_trigger is not None:
                         nursery.start_soon(raise_shutdown, shutdown_trigger)
+                    nursery.start_soon(raise_shutdown, context.terminate.wait)
 
                     nursery.start_soon(
                         partial(
@@ -89,8 +98,10 @@ async def worker_serve(
                     )
 
                     await trio.sleep_forever()
-            except (ShutdownError, KeyboardInterrupt):
-                pass
+            except BaseExceptionGroup as error:
+                _, other_errors = error.split((ShutdownError, KeyboardInterrupt))
+                if other_errors is not None:
+                    raise other_errors
             finally:
                 await context.terminated.set()
                 server_nursery.cancel_scope.deadline = trio.current_time() + config.graceful_timeout
